import {
  Actions,
  ActionType,
} from './discover.action';

import {
  State,
  initialState,
} from './discover.state';

import {
  Zone,
  Host,
  Port,
  Service,
} from '../../model';

// import * as _ 'lodash';

export function reducer(state = initialState, action: Actions): State {
  switch (action.type) {
    case ActionType.DiscoveredZone: {
      const zone: Zone = <Zone>action.payload;
      const zones: Map<string, Zone> = null === state.zones ? new Map() : state.zones;

      zones.set(zone.network, zone);

      const newZones: Map<string, Zone> = new Map();

      zones.forEach(function(value, key) {
        newZones.set(key, value);
      });

      return {
        ...state,
        zones : newZones,
      };
    }

    case ActionType.DiscoveredHost: {
      const host: Host = <Host>action.payload;
      let zone = null;

      let zones: Map<string, Zone> = state.zones;

      if (zones === undefined || zones === null) {
        zones = new Map();
        zone = host.zone;
        // zones.set(zone.network, zone);
      } else {
        zone = zones.get(host.zone.network);
      }

      if (undefined === zone) {
        console.error(`Discovery.discoveredHost: Zone[${host.zone.network}] is not exist`);
      }
      if (null === zone.hosts || undefined === zone.hosts) {
        zone.hosts = new Map();
      }
      zone.hosts.set(host.ip, host);
      zones.set(zone.network, zone);

      const newZones: Map<string, Zone> = new Map();

      zones.forEach(function(value, key) {
        newZones.set(key, value);
      });

      return {
        ...state,
        zones: newZones
      };
    }

    case ActionType.DiscoveredPort: {
      const port: Port = <Port>action.payload;
      const zone = state.zones.get(port.host.zone.network);
      if (undefined === zone) {
        console.error(`Discovery.DiscoveredPort: Zone[${port.host.zone.network}] is not exist`);
      }
      if (null === zone.hosts || undefined === zone.hosts.get(port.host.ip)) {
        console.error(`Discovery.DiscoveredPort: Host[${port.host.ip}] is not exist`);
      }
      const host: Host = zone.hosts.get(port.host.ip);

      if (null === host.ports) {
        host.ports = new Map();
      }
      host.ports.set(port.portNumber, port);
      return state;
    }

    case ActionType.DiscoveredService: {
      const service: Service = <Service>action.payload;
      const zone = state.zones.get(service.port.host.zone.network);
      if (undefined === zone) {
        console.error(`Discovery.DiscoveredService: Zone[${service.port.host.zone.network}] is not exist`);
      }
      if (null === zone.hosts || undefined === zone.hosts.get(service.port.host.ip)) {
        console.error(`Discovery.DiscoveredPort: Host[${service.port.host.ip}] is not exist`);
      }
      const host: Host = zone.hosts.get(service.port.host.ip);
      if (null === host.ports || undefined === host.ports.get(service.port.portNumber)) {
        console.error(`Discovery.DiscoveredPort: Port[${service.port.portNumber}] is not exist`);
      }
      const port: Port = host.ports.get(service.port.portNumber);

      if (null === port.services) {
        port.services = new Map();
      }
      port.services.set(service.serviceName, service);
      return state;
    }

    default: {
      return state;
    }
  }
}
